.admin-ui-page{
  background-color:#fff;
  color:#2f2f2f;
  container:admin-ui-page/inline-size;
  display:flex;
  flex-flow:column;
  height:100%;
  position:relative;
  z-index:1;
}
@media not (prefers-reduced-motion){
  .admin-ui-page{
    transition:width .2s ease-out;
  }
}

.admin-ui-page__header{
  background:#fff;
  border-bottom:1px solid #f0f0f0;
  padding:16px 48px;
  position:sticky;
  top:0;
}
@container (max-width: 430px){
  .admin-ui-page__header{
    padding:16px 24px;
  }
}

.admin-ui-page__header-subtitle{
  color:#757575;
  font-family:-apple-system,"system-ui",Segoe UI,Roboto,Oxygen-Sans,Ubuntu,Cantarell,Helvetica Neue,sans-serif;
  font-size:13px;
  font-weight:400;
  line-height:20px;
  margin:0;
  padding-block-end:8px;
}

.admin-ui-page__content{
  display:flex;
  flex-direction:column;
  flex-grow:1;
  overflow:auto;
}
.admin-ui-page__content.has-padding{
  padding:16px 20px;
}
@container (max-width: 430px){
  .admin-ui-page__content.has-padding{
    padding:16px 24px;
  }
}

.show-icon-labels .admin-ui-page__header-actions .components-button.has-icon{
  padding:0 8px;
  width:auto;
}
.show-icon-labels .admin-ui-page__header-actions .components-button.has-icon svg{
  display:none;
}
.show-icon-labels .admin-ui-page__header-actions .components-button.has-icon:after{
  content:attr(aria-label);
  font-size:12px;
}