<?php
// This file is automatically generated. Do not edit directly.
if ( ! defined( 'ABSPATH' ) ) {
	die( 'Silence is golden.' );
}

return array(
	'arrow-down-left'     => array(
		'label'    => _x( 'Arrow Down Left', 'icon label' ),
		'filePath' => 'arrow-down-left.svg',
	),
	'arrow-down-right'    => array(
		'label'    => _x( 'Arrow Down Right', 'icon label' ),
		'filePath' => 'arrow-down-right.svg',
	),
	'arrow-down'          => array(
		'label'    => _x( 'Arrow Down', 'icon label' ),
		'filePath' => 'arrow-down.svg',
	),
	'arrow-left'          => array(
		'label'    => _x( 'Arrow Left', 'icon label' ),
		'filePath' => 'arrow-left.svg',
	),
	'arrow-right'         => array(
		'label'    => _x( 'Arrow Right', 'icon label' ),
		'filePath' => 'arrow-right.svg',
	),
	'arrow-up-left'       => array(
		'label'    => _x( 'Arrow Up Left', 'icon label' ),
		'filePath' => 'arrow-up-left.svg',
	),
	'arrow-up-right'      => array(
		'label'    => _x( 'Arrow Up Right', 'icon label' ),
		'filePath' => 'arrow-up-right.svg',
	),
	'arrow-up'            => array(
		'label'    => _x( 'Arrow Up', 'icon label' ),
		'filePath' => 'arrow-up.svg',
	),
	'at-symbol'           => array(
		'label'    => _x( 'At Symbol (@)', 'icon label' ),
		'filePath' => 'at-symbol.svg',
	),
	'audio'               => array(
		'label'    => _x( 'Audio', 'icon label' ),
		'filePath' => 'audio.svg',
	),
	'bell'                => array(
		'label'    => _x( 'Bell', 'icon label' ),
		'filePath' => 'bell.svg',
	),
	'block-default'       => array(
		'label'    => _x( 'Block Default', 'icon label' ),
		'filePath' => 'block-default.svg',
	),
	'block-meta'          => array(
		'label'    => _x( 'Block Meta', 'icon label' ),
		'filePath' => 'block-meta.svg',
	),
	'block-table'         => array(
		'label'    => _x( 'Block Table', 'icon label' ),
		'filePath' => 'block-table.svg',
	),
	'calendar'            => array(
		'label'    => _x( 'Calendar', 'icon label' ),
		'filePath' => 'calendar.svg',
	),
	'capture-photo'       => array(
		'label'    => _x( 'Capture Photo', 'icon label' ),
		'filePath' => 'capture-photo.svg',
	),
	'capture-video'       => array(
		'label'    => _x( 'Capture Video', 'icon label' ),
		'filePath' => 'capture-video.svg',
	),
	'cart'                => array(
		'label'    => _x( 'Cart', 'icon label' ),
		'filePath' => 'cart.svg',
	),
	'category'            => array(
		'label'    => _x( 'Category', 'icon label' ),
		'filePath' => 'category.svg',
	),
	'caution'             => array(
		'label'    => _x( 'Caution', 'icon label' ),
		'filePath' => 'caution.svg',
	),
	'chart-bar'           => array(
		'label'    => _x( 'Chart Bar', 'icon label' ),
		'filePath' => 'chart-bar.svg',
	),
	'check'               => array(
		'label'    => _x( 'Check', 'icon label' ),
		'filePath' => 'check.svg',
	),
	'chevron-down'        => array(
		'label'    => _x( 'Chevron Down', 'icon label' ),
		'filePath' => 'chevron-down.svg',
	),
	'chevron-down-small'  => array(
		'label'    => _x( 'Chevron Down Small', 'icon label' ),
		'filePath' => 'chevron-down-small.svg',
	),
	'chevron-left'        => array(
		'label'    => _x( 'Chevron Left', 'icon label' ),
		'filePath' => 'chevron-left.svg',
	),
	'chevron-left-small'  => array(
		'label'    => _x( 'Chevron Left Small', 'icon label' ),
		'filePath' => 'chevron-left-small.svg',
	),
	'chevron-right'       => array(
		'label'    => _x( 'Chevron Right', 'icon label' ),
		'filePath' => 'chevron-right.svg',
	),
	'chevron-right-small' => array(
		'label'    => _x( 'Chevron Right Small', 'icon label' ),
		'filePath' => 'chevron-right-small.svg',
	),
	'chevron-up'          => array(
		'label'    => _x( 'Chevron Up', 'icon label' ),
		'filePath' => 'chevron-up.svg',
	),
	'chevron-up-down'     => array(
		'label'    => _x( 'Chevron Up Down', 'icon label' ),
		'filePath' => 'chevron-up-down.svg',
	),
	'chevron-up-small'    => array(
		'label'    => _x( 'Chevron Up Small', 'icon label' ),
		'filePath' => 'chevron-up-small.svg',
	),
	'comment'             => array(
		'label'    => _x( 'Comment', 'icon label' ),
		'filePath' => 'comment.svg',
	),
	'cover'               => array(
		'label'    => _x( 'Cover', 'icon label' ),
		'filePath' => 'cover.svg',
	),
	'create'              => array(
		'label'    => _x( 'Create', 'icon label' ),
		'filePath' => 'create.svg',
	),
	'desktop'             => array(
		'label'    => _x( 'Desktop', 'icon label' ),
		'filePath' => 'desktop.svg',
	),
	'download'            => array(
		'label'    => _x( 'Download', 'icon label' ),
		'filePath' => 'download.svg',
	),
	'drawer-left'         => array(
		'label'    => _x( 'Drawer Left', 'icon label' ),
		'filePath' => 'drawer-left.svg',
	),
	'drawer-right'        => array(
		'label'    => _x( 'Drawer Right', 'icon label' ),
		'filePath' => 'drawer-right.svg',
	),
	'envelope'            => array(
		'label'    => _x( 'Envelope', 'icon label' ),
		'filePath' => 'envelope.svg',
	),
	'error'               => array(
		'label'    => _x( 'Error', 'icon label' ),
		'filePath' => 'error.svg',
	),
	'external'            => array(
		'label'    => _x( 'External', 'icon label' ),
		'filePath' => 'external.svg',
	),
	'file'                => array(
		'label'    => _x( 'File', 'icon label' ),
		'filePath' => 'file.svg',
	),
	'gallery'             => array(
		'label'    => _x( 'Gallery', 'icon label' ),
		'filePath' => 'gallery.svg',
	),
	'group'               => array(
		'label'    => _x( 'Group', 'icon label' ),
		'filePath' => 'group.svg',
	),
	'heading'             => array(
		'label'    => _x( 'Heading', 'icon label' ),
		'filePath' => 'heading.svg',
	),
	'help'                => array(
		'label'    => _x( 'Help', 'icon label' ),
		'filePath' => 'help.svg',
	),
	'home'                => array(
		'label'    => _x( 'Home', 'icon label' ),
		'filePath' => 'home.svg',
	),
	'image'               => array(
		'label'    => _x( 'Image', 'icon label' ),
		'filePath' => 'image.svg',
	),
	'info'                => array(
		'label'    => _x( 'Info', 'icon label' ),
		'filePath' => 'info.svg',
	),
	'key'                 => array(
		'label'    => _x( 'Key', 'icon label' ),
		'filePath' => 'key.svg',
	),
	'language'            => array(
		'label'    => _x( 'Language', 'icon label' ),
		'filePath' => 'language.svg',
	),
	'map-marker'          => array(
		'label'    => _x( 'Map Marker', 'icon label' ),
		'filePath' => 'map-marker.svg',
	),
	'menu'                => array(
		'label'    => _x( 'Menu', 'icon label' ),
		'filePath' => 'menu.svg',
	),
	'mobile'              => array(
		'label'    => _x( 'Mobile', 'icon label' ),
		'filePath' => 'mobile.svg',
	),
	'more-horizontal'     => array(
		'label'    => _x( 'More Horizontal', 'icon label' ),
		'filePath' => 'more-horizontal.svg',
	),
	'more-vertical'       => array(
		'label'    => _x( 'More Vertical', 'icon label' ),
		'filePath' => 'more-vertical.svg',
	),
	'next'                => array(
		'label'    => _x( 'Next', 'icon label' ),
		'filePath' => 'next.svg',
	),
	'paragraph'           => array(
		'label'    => _x( 'Paragraph', 'icon label' ),
		'filePath' => 'paragraph.svg',
	),
	'payment'             => array(
		'label'    => _x( 'Payment', 'icon label' ),
		'filePath' => 'payment.svg',
	),
	'pencil'              => array(
		'label'    => _x( 'Pencil', 'icon label' ),
		'filePath' => 'pencil.svg',
	),
	'people'              => array(
		'label'    => _x( 'People', 'icon label' ),
		'filePath' => 'people.svg',
	),
	'plus'                => array(
		'label'    => _x( 'Plus', 'icon label' ),
		'filePath' => 'plus.svg',
	),
	'plus-circle'         => array(
		'label'    => _x( 'Plus Circle', 'icon label' ),
		'filePath' => 'plus-circle.svg',
	),
	'previous'            => array(
		'label'    => _x( 'Previous', 'icon label' ),
		'filePath' => 'previous.svg',
	),
	'published'           => array(
		'label'    => _x( 'Published', 'icon label' ),
		'filePath' => 'published.svg',
	),
	'quote'               => array(
		'label'    => _x( 'Quote', 'icon label' ),
		'filePath' => 'quote.svg',
	),
	'receipt'             => array(
		'label'    => _x( 'Receipt', 'icon label' ),
		'filePath' => 'receipt.svg',
	),
	'rss'                 => array(
		'label'    => _x( 'RSS', 'icon label' ),
		'filePath' => 'rss.svg',
	),
	'scheduled'           => array(
		'label'    => _x( 'Scheduled', 'icon label' ),
		'filePath' => 'scheduled.svg',
	),
	'search'              => array(
		'label'    => _x( 'Search', 'icon label' ),
		'filePath' => 'search.svg',
	),
	'settings'            => array(
		'label'    => _x( 'Settings', 'icon label' ),
		'filePath' => 'settings.svg',
	),
	'shadow'              => array(
		'label'    => _x( 'Shadow', 'icon label' ),
		'filePath' => 'shadow.svg',
	),
	'share'               => array(
		'label'    => _x( 'Share', 'icon label' ),
		'filePath' => 'share.svg',
	),
	'shield'              => array(
		'label'    => _x( 'Shield', 'icon label' ),
		'filePath' => 'shield.svg',
	),
	'shuffle'             => array(
		'label'    => _x( 'Shuffle', 'icon label' ),
		'filePath' => 'shuffle.svg',
	),
	'star-empty'          => array(
		'label'    => _x( 'Star Empty', 'icon label' ),
		'filePath' => 'star-empty.svg',
	),
	'star-filled'         => array(
		'label'    => _x( 'Star Filled', 'icon label' ),
		'filePath' => 'star-filled.svg',
	),
	'star-half'           => array(
		'label'    => _x( 'Star Half', 'icon label' ),
		'filePath' => 'star-half.svg',
	),
	'store'               => array(
		'label'    => _x( 'Store', 'icon label' ),
		'filePath' => 'store.svg',
	),
	'styles'              => array(
		'label'    => _x( 'Styles', 'icon label' ),
		'filePath' => 'styles.svg',
	),
	'symbol'              => array(
		'label'    => _x( 'Symbol', 'icon label' ),
		'filePath' => 'symbol.svg',
	),
	'symbol-filled'       => array(
		'label'    => _x( 'Symbol Filled', 'icon label' ),
		'filePath' => 'symbol-filled.svg',
	),
	'table'               => array(
		'label'    => _x( 'Table', 'icon label' ),
		'filePath' => 'table.svg',
	),
	'tablet'              => array(
		'label'    => _x( 'Tablet', 'icon label' ),
		'filePath' => 'tablet.svg',
	),
	'tag'                 => array(
		'label'    => _x( 'Tag', 'icon label' ),
		'filePath' => 'tag.svg',
	),
	'tip'                 => array(
		'label'    => _x( 'Tip', 'icon label' ),
		'filePath' => 'tip.svg',
	),
	'upload'              => array(
		'label'    => _x( 'Upload', 'icon label' ),
		'filePath' => 'upload.svg',
	),
	'verse'               => array(
		'label'    => _x( 'Verse', 'icon label' ),
		'filePath' => 'verse.svg',
	),
);
