<?php
/**
 * Pages loader - Auto-generated by build process.
 * Do not edit this file manually.
 *
 * @package wp
 */

require_once __DIR__ . '/pages/font-library/page.php';
require_once __DIR__ . '/pages/font-library/page-wp-admin.php';
require_once __DIR__ . '/pages/options-connectors/page.php';
require_once __DIR__ . '/pages/options-connectors/page-wp-admin.php';
