.wp-block-latest-posts > li {
  overflow: hidden;
}

.wp-block-latest-posts li a > div {
  display: inline;
}

:root :where(.wp-block-latest-posts) {
  padding-left: 2.5em;
}

:root :where(.wp-block-latest-posts.is-grid),
:root :where(.wp-block-latest-posts__list) {
  padding-left: 0;
}