.wp-block[data-align=center] > .wp-block-query-pagination {
  justify-content: center;
}

:where(.editor-styles-wrapper) .wp-block-query-pagination {
  max-width: 100%;
}
:where(.editor-styles-wrapper) .wp-block-query-pagination.block-editor-block-list__layout {
  margin: 0;
}