/**
 * Breakpoints & Media Queries
 */
/**
 * Typography
 */
/**
 * SCSS Variables.
 *
 * Please use variables from this sheet to ensure consistency across the UI.
 * Don't add to this sheet unless you're pretty sure the value will be reused in many places.
 * For example, don't add rules to this sheet that affect block visuals. It's purely for UI.
 */
/**
 * Colors
 */
/**
 * Fonts & basic variables.
 */
/**
 * Typography
 */
/**
 * Grid System.
 * https://make.wordpress.org/design/2019/10/31/proposal-a-consistent-spacing-system-for-wordpress/
 */
/**
 * Radius scale.
 */
/**
 * Elevation scale.
 */
/**
 * Dimensions.
 */
/**
 * Mobile specific styles
 */
/**
 * Editor styles.
 */
/**
 * Block & Editor UI.
 */
/**
 * Block paddings.
 */
/**
 * React Native specific.
 * These variables do not appear to be used anywhere else.
 */
/**
*  Converts a hex value into the rgb equivalent.
*
* @param {string} hex - the hexadecimal value to convert
* @return {string} comma separated rgb values
*/
/**
 * Long content fade mixin
 *
 * Creates a fading overlay to signify that the content is longer
 * than the space allows.
 */
/**
 * Breakpoint mixins
 */
/**
 * Focus styles.
 */
/**
 * Applies editor left position to the selector passed as argument
 */
/**
 * Styles that are reused verbatim in a few places
 */
/**
 * Allows users to opt-out of animations via OS-level preferences.
 */
/**
 * Reset default styles for JavaScript UI based pages.
 * This is a WP-admin agnostic reset
 */
/**
 * Reset the WP Admin page styles for Gutenberg-like pages.
 */
/**
 * Creates a checkerboard pattern background to indicate transparency.
 * @param {String} $size - The size of the squares in the checkerboard pattern. Default is 12px.
 */
.nux-dot-tip::before, .nux-dot-tip::after {
  border-radius: 100%;
  content: " ";
  pointer-events: none;
  position: absolute;
}
.nux-dot-tip::before {
  background: rgba(0, 115, 156, 0.9);
  opacity: 0.9;
  height: 24px;
  right: -12px;
  top: -12px;
  transform: scale(0.3333333333);
  width: 24px;
}
@media not (prefers-reduced-motion) {
  .nux-dot-tip::before {
    animation: nux-pulse 1.6s infinite cubic-bezier(0.17, 0.67, 0.92, 0.62);
  }
}
.nux-dot-tip::after {
  background: #00739c;
  height: 8px;
  right: -4px;
  top: -4px;
  width: 8px;
}
@keyframes nux-pulse {
  100% {
    background: rgba(0, 115, 156, 0);
    transform: scale(1);
  }
}
.nux-dot-tip .components-popover__content {
  width: 350px;
  padding: 20px 18px;
}
@media (min-width: 600px) {
  .nux-dot-tip .components-popover__content {
    width: 450px;
  }
}
.nux-dot-tip .components-popover__content .nux-dot-tip__disable {
  position: absolute;
  left: 0;
  top: 0;
}
.nux-dot-tip[data-y-axis=top] {
  margin-top: -4px;
}
.nux-dot-tip[data-y-axis=bottom] {
  margin-top: 4px;
}
.nux-dot-tip[data-y-axis=middle][data-y-axis=left] {
  margin-right: -4px;
}
.nux-dot-tip[data-y-axis=middle][data-y-axis=right] {
  margin-right: 4px;
}
.nux-dot-tip[data-y-axis=top] .components-popover__content {
  margin-bottom: 20px;
}
.nux-dot-tip[data-y-axis=bottom] .components-popover__content {
  margin-top: 20px;
}
.nux-dot-tip[data-y-axis=middle][data-y-axis=left] .components-popover__content {
  margin-left: 20px;
}
.nux-dot-tip[data-y-axis=middle][data-y-axis=right] .components-popover__content {
  margin-right: 20px;
}
.nux-dot-tip[data-y-axis=left], .nux-dot-tip[data-y-axis=center], .nux-dot-tip[data-y-axis=right] {
  z-index: 1000001;
}
@media (max-width: 600px) {
  .nux-dot-tip[data-y-axis=left] .components-popover__content, .nux-dot-tip[data-y-axis=center] .components-popover__content, .nux-dot-tip[data-y-axis=right] .components-popover__content {
    align-self: end;
    right: 5px;
    margin: 20px 0 0 0;
    max-width: none !important;
    position: fixed;
    left: 5px;
    width: auto;
  }
}
.nux-dot-tip.components-popover:not([data-y-axis=middle])[data-y-axis=right] .components-popover__content {
  margin-left: 0;
}
.nux-dot-tip.components-popover:not([data-y-axis=middle])[data-y-axis=left] .components-popover__content {
  margin-right: 0;
}
.nux-dot-tip.components-popover.interface-more-menu-dropdown__content:not([data-y-axis=middle])[data-y-axis=right] .components-popover__content {
  margin-left: -12px;
}
.nux-dot-tip.components-popover.interface-more-menu-dropdown__content:not([data-y-axis=middle])[data-y-axis=left] .components-popover__content {
  margin-right: -12px;
}